from typing import Callable

from guet.commands.strategy import CommandStrategy


class LambdaStrategy(CommandStrategy):

    def __init__(self, apply: Callable):
        self._apply = apply

    def apply(self):
        self._apply()



from typing import List

from guet.git._guet_hooks import GUET_HOOKS
from guet.git.hook import Hook


def _name(hook: Hook) -> str:
    return hook.path.split('/')[-1]


def _normal_hooks(hooks: List[Hook]) -> List[Hook]:
    return [hook for hook in hooks if _name(hook) in GUET_HOOKS]


def _dash_guet_normal_hooks(hooks: List[Hook]) -> List[Hook]:
    final = []
    for hook in hooks:
        name = _name(hook)
        if name.endswith('-guet') and name.replace('-guet', '') in GUET_HOOKS:
            final.append(hook)
    return final


def all_valid_hooks(hooks: List[Hook]) -> bool:
    hook_names = [_name(hook) for hook in _normal_hooks(hooks)]
    valid_names = GUET_HOOKS == hook_names
    valid_content = all([hook.is_guet_hook() for hook in _normal_hooks(hooks)])
    if not (valid_names and valid_content):
        hook_names = [_name(hook).replace('-guet', '') for hook in _dash_guet_normal_hooks(hooks)]
        valid_names = GUET_HOOKS == hook_names
        valid_content = all([hook.is_guet_hook() for hook in _dash_guet_normal_hooks(hooks)])
        return valid_names and valid_content
    return True



from typing import List

from guet.commands.command import Command
from guet.commands.command_factory_decorator import CommandFactoryDecorator
from guet.commands.do_nothing_strategy import DoNothingStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.config.already_initialized import already_initialized
from guet.settings.settings import Settings

INIT_REQUIRED_ERROR_MESSAGE = ('guet has not been initialized yet! ' +
                               'Please do so by running the command "guet init".')


class InitRequiredDecorator(CommandFactoryDecorator):
    def build(self, args: List[str], settings: Settings) -> Command:
        if already_initialized():
            return self.decorated.build(args, settings)
        else:
            print(INIT_REQUIRED_ERROR_MESSAGE)
            exit(1)
            return StrategyCommand(DoNothingStrategy())



from typing import List

from guet.commands.command import Command
from guet.commands.command_factory import CommandFactoryMethod
from guet.settings.settings import Settings


class CommandFactoryDecorator(CommandFactoryMethod):
    def __init__(self, decorated: CommandFactoryMethod):
        super().__init__()
        self.decorated = decorated

    def short_help_message(self):
        return self.decorated.short_help_message()

    def build(self, args: List[str], settings: Settings) -> Command:
        raise NotImplementedError



from typing import List

from guet.commands.strategy import CommandStrategy
from guet.config.committer import filter_committers_with_initials, Committer
from guet.context.context import Context


class SetCommittersStrategy(CommandStrategy):

    def __init__(self, args: List[str], context: Context):
        self.args = args
        self.context = context

    def apply(self):
        committers = self.context.committers.all()
        committer_initials = self.args
        committers_to_set = filter_committers_with_initials(committers, committer_initials)

        correct_number_of_committers_present = len(committers_to_set) is len(committer_initials)

        if not correct_number_of_committers_present:
            for initials in committer_initials:
                if not self._committer_with_initials_present(committers, initials):
                    print(f"No committer exists with initials '{initials}'")
        else:
            self.context.set_committers(committers_to_set)

    def _committer_with_initials_present(self, committers: List[Committer], initials: str):
        committer_with_initial_present = False
        for committer in committers:
            if committer.initials == initials:
                committer_with_initial_present = True
        return committer_with_initial_present



from os.path import join

from guet import constants
from guet.config import CONFIGURATION_DIRECTORY
from guet.config.errors import PairSetError
from guet.files.read_lines import read_lines
from guet.config.parse_comitters_set_line import parse_committers_set_line
from guet.git.git_path_from_cwd import git_path_from_cwd


def most_recent_committers_set():
    path = join(CONFIGURATION_DIRECTORY, constants.COMMITTERS_SET)
    lines = read_lines(path)
    try:
        line = next(line for line in lines if line.endswith(git_path_from_cwd()))
        *_, set_time, _ = parse_committers_set_line(line)
        return int(set_time)
    except StopIteration:
        raise PairSetError()



from typing import List

from guet.commands.addcommitter.add_committer_strategy import AddCommitterStrategy
from guet.commands.cancellable_strategy import CancelableCommandStrategy
from guet.commands.command import Command
from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.do_nothing_strategy import DoNothingStrategy
from guet.commands.help.help_message_builder import HelpMessageBuilder
from guet.commands.strategy import CommandStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.commands.too_few_args import TooFewArgsStrategy
from guet.commands.too_many_args import TooManyArgsStrategy
from guet.settings.settings import Settings

ADD_COMMITTER_HELP_MESSAGE = HelpMessageBuilder('guet add <initials> <"name"> <email>',
                                                "Add committer to make available for commit tracking.").build()


class AddCommitterFactory(CommandFactoryMethod):
    def build(self, args: List[str], settings: Settings) -> Command:
        strategy = self._choose_strategy(args[1:], settings)
        return StrategyCommand(strategy)

    def short_help_message(self):
        return 'Add committer to the list of available committers'

    def _initials_already_present(self, initials):
        return len([committer for committer in self.context.committers.all() if committer.initials == initials]) != 0

    def _prompt(self, new_initials: str, new_name: str, new_email: str) -> str:
        matching_committer = next(
            (committer for committer in self.context.committers.all() if committer.initials == new_initials))
        warning = (f'Matching initials "{new_initials}". Adding "{new_name}" <{new_email}> '
                   f'will overwrite "{matching_committer.name}" <{matching_committer.email}>. Would you '
                   f'like to continue (y) or cancel (x)?')
        return warning

    def _choose_strategy(self, args: List[str], _: Settings) -> CommandStrategy:
        if len(args) < 3:
            return TooFewArgsStrategy(ADD_COMMITTER_HELP_MESSAGE)
        elif len(args) > 3:
            return TooManyArgsStrategy()
        else:
            initials, name, email = args
            add_committers_strategy = AddCommitterStrategy(initials, name, email)
            if self._initials_already_present(initials):
                return CancelableCommandStrategy(self._prompt(initials, name, email),
                                                 add_committers_strategy,
                                                 DoNothingStrategy())
            else:
                return add_committers_strategy



import traceback

from guet.config.set_errors import set_errors


def log_on_error(wrapped):
    def wrapper():
        try:
            wrapped()
        # pylint: disable=broad-except
        except Exception:
            print('An error has occurred, please refer to error logs (~/.guet/errors) for more information\n')
            stack_tract: str = traceback.format_exc()
            set_errors(stack_tract.split('\n'))
            exit(1)

    return wrapper



from os.path import join, isdir
from typing import List

from guet.config.committer import Committer
from guet.context.set_committer_observer import SetCommitterObserver
from guet.files.read_lines import read_lines
from guet.files.write_lines import write_lines
from guet.git._all_valid_hooks import all_valid_hooks
from guet.git._author_manage import load_author, overwrite_current_author, get_author_lines, append_new_author
from guet.git._create_strategy import DoCreateStrategy, DontCreateStrategy
from guet.git._file_name_strategy import BaseFileNameStrategy, AlongsideFileNameStrategy
from guet.git._guet_hooks import GUET_HOOKS
from guet.git._hook_loader import HookLoader
from guet.git.author import Author
from guet.git.errors import NoGitPresentError
from guet.git.hook import Hook


def _load_hooks(hook_strategy: HookLoader) -> List[Hook]:
    hooks = []
    for hook_name in GUET_HOOKS:
        hook_strategy.apply(hook_name, hooks)
    return hooks


def _load_commit_msg(path_to_repository) -> List[str]:
    try:
        return read_lines(join(path_to_repository, 'COMMIT_EDITMSG'))
    except FileNotFoundError:
        return []


class Git(SetCommitterObserver):

    def __init__(self, repository_path: str):
        super().__init__()
        if not isdir(repository_path):
            raise NoGitPresentError()
        default = _load_hooks(HookLoader(repository_path, BaseFileNameStrategy(), DontCreateStrategy()))
        alongside = _load_hooks(HookLoader(repository_path, AlongsideFileNameStrategy(), DontCreateStrategy()))
        self.hooks = default + alongside
        self.path_to_repository = repository_path
        self._commit_msg = _load_commit_msg(repository_path)
        self._config_lines = read_lines(join(repository_path, 'config'))
        self._author: Author = load_author(self._config_lines)

    @property
    def author(self) -> Author:
        return self._author

    @author.setter
    def author(self, new_author: Author):
        new_lines = list(self._config_lines)
        user, email = get_author_lines(self._config_lines)
        if user is not None and email is not None:
            overwrite_current_author(new_lines, new_author)
        else:
            append_new_author(new_lines, new_author)
        write_lines(join(self.path_to_repository, 'config'), new_lines)
        self._author = new_author

    @property
    def commit_msg(self):
        return self._commit_msg

    @commit_msg.setter
    def commit_msg(self, lines: List[str]):
        write_lines(join(self.path_to_repository, 'COMMIT_EDITMSG'), lines)
        self._commit_msg = lines

    def hooks_present(self) -> bool:
        return all_valid_hooks(self.hooks)

    def non_guet_hooks_present(self) -> bool:
        non_guet_hook_found = False
        for hook in self.hooks:
            if not hook.is_guet_hook():
                non_guet_hook_found = True
        return non_guet_hook_found

    def create_hooks(self, alongside: bool = False) -> None:
        if alongside:
            hook_loader = HookLoader(self.path_to_repository, AlongsideFileNameStrategy(), DoCreateStrategy())
        else:
            hook_loader = HookLoader(self.path_to_repository, BaseFileNameStrategy(), DoCreateStrategy())
        self.hooks = _load_hooks(hook_loader)
        for hook in self.hooks:
            hook.save()

    def notify_of_committer_set(self, new_committers: List[Committer]):
        author_committer = new_committers[0]
        self.author = Author(name=author_committer.name, email=author_committer.email)



from typing import List

from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.help.help_message_builder import HelpMessageBuilder
from guet.commands.setcommitters.set_committers_strategy import SetCommittersStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.settings.settings import Settings

SET_HELP_MESSAGE = HelpMessageBuilder('guet set <initials> [<initials> ...]', 'Get current committers.').build()


class SetCommittersCommandFactory(CommandFactoryMethod):
    def build(self, args: List[str], settings: Settings):
        return StrategyCommand(SetCommittersStrategy(args[1:], self.context))

    def short_help_message(self):
        return 'Set the current committers'



from os import chmod, stat

from guet.files.read_lines import read_lines
from guet.files.write_lines import write_lines

GUET_HOOK_FILE = [
    '#! /usr/bin/env python3',
    'from guet.hooks import manage',
    'import sys',
    'manage(sys.argv[0])',
]


class Hook:
    def __init__(self, path_to_hook: str, *, create: bool = False):
        self.path = path_to_hook
        self.content = self._parse_file_content(create, path_to_hook)

    def __repr__(self) -> str:
        return f'Hook: path: {self.path}, content: {self.content}'

    def is_guet_hook(self):
        return self.content == GUET_HOOK_FILE

    @staticmethod
    def _parse_file_content(create, path_to_hook):
        _content = Hook._get_file_content(path_to_hook, create)
        if _content != GUET_HOOK_FILE:
            _content = Hook._handle_mismatched_content(_content, create)
        return _content

    @staticmethod
    def _handle_mismatched_content(_content, create):
        if create:
            _content = GUET_HOOK_FILE
        return _content

    @staticmethod
    def _get_file_content(path_to_hook, create):
        try:
            _content = read_lines(path_to_hook)
        except FileNotFoundError:
            if create:
                _content = GUET_HOOK_FILE
            else:
                raise
        return _content

    def save(self):
        write_lines(self.path, self.content)
        status = stat(self.path)
        chmod(self.path, status.st_mode | 0o111)



from typing import List

from guet.git.git import Git
from guet.git.git_path_from_cwd import git_path_from_cwd

from guet.config.committer import Committer
from guet.config.get_current_committers import get_current_committers
from guet.util.errors import log_on_error


def _create_co_authored_line(committer: Committer):
    return f'Co-authored-by: {committer.name} <{committer.email}>\n'


def _replace_already_present_co_authored_messages(commit_message: List[str],
                                                  co_authored_lines: List[str]):
    commit_message_without_co_authored = [
        line for line in commit_message if not line.startswith('Co-authored')
    ]
    _remove_trailing_newline(commit_message_without_co_authored)
    return commit_message_without_co_authored + ['\n'] + co_authored_lines


def _remove_trailing_newline(commit_message_without_co_authored):
    if commit_message_without_co_authored[len(commit_message_without_co_authored) - 1] == '\n':
        del commit_message_without_co_authored[len(commit_message_without_co_authored) - 1]


@log_on_error
def commit_msg():
    git_path = git_path_from_cwd()
    git = Git(git_path)
    commit_message = git.commit_msg
    current_committers = get_current_committers()
    co_authored_lines = [_create_co_authored_line(committer) for committer in current_committers]
    final_commit_message = _replace_already_present_co_authored_messages(
        commit_message, co_authored_lines)
    git.commit_msg = final_commit_message



from typing import List, Callable

from guet.commands.strategy import CommandStrategy
from guet.config.committer import Committer


class CommitterPrintingStrategy(CommandStrategy):
    def __init__(self,
                 committers: List[Committer],
                 pre_print_strategy: Callable[[], None],
                 listing_strategy: Callable[[List[Committer]], None]):
        self.committers = committers
        self.pre_print_strategy = pre_print_strategy
        self.listing_strategy = listing_strategy

    def apply(self):
        self.pre_print_strategy()
        self.listing_strategy(self.committers)



from typing import List

from guet.commands.command import Command
from guet.commands.command_factory_decorator import CommandFactoryDecorator
from guet.commands.print_strategy import PrintCommandStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.settings.settings import Settings
import guet


class VersionDecorator(CommandFactoryDecorator):
    def build(self, args: List[str], settings: Settings) -> Command:
        if '-v' in args or '--version' in args:
            return StrategyCommand(PrintCommandStrategy(guet.__version__))
        else:
            return self.decorated.build(args, settings)



from guet.commands.strategy import CommandStrategy


class PrintCommandStrategy(CommandStrategy):
    def __init__(self, text: str):
        self._text = text

    def apply(self):
        print(self._text)



from os import getcwd
from os.path import join
from typing import List

from guet.commands.command import Command
from guet.commands.command_factory_decorator import CommandFactoryDecorator
from guet.commands.print_strategy import PrintCommandStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.git.errors import NoGitPresentError
from guet.git.git import Git
from guet.settings.settings import Settings


class GitRequiredDecorator(CommandFactoryDecorator):
    def build(self, args: List[str], settings: Settings) -> Command:
        try:
            Git(join(getcwd(), '.git'))
            return self.decorated.build(args, settings)
        except NoGitPresentError:
            return StrategyCommand(PrintCommandStrategy('Git not initialized in this directory.'))



from os.path import join
from typing import List

from guet import constants
from guet.config import CONFIGURATION_DIRECTORY

_COMMITTER_NOT_PRESENT = -1


def add_committer(initials: str, name: str, email: str) -> None:
    all_committers = _read_all_committers_from_file()
    _add_committer_to_committers(all_committers, initials, name, email)
    _write_committers_to_file(all_committers)


def _add_committer_to_committers(all_committers: List[str], initials: str, name: str, email: str):
    committer_formatted = f'{initials},{name},{email}\n'
    committer_position = _position_of_committer_with_initials(all_committers, initials)
    if committer_position is _COMMITTER_NOT_PRESENT:
        all_committers.append(committer_formatted)
    else:
        all_committers[committer_position] = committer_formatted


def _position_of_committer_with_initials(all_committers: List[str], initials: str) -> int:
    for index, committer in enumerate(all_committers):
        if committer.startswith(initials):
            return index
    return _COMMITTER_NOT_PRESENT


def _read_all_committers_from_file() -> List[str]:
    committers_file = open(join(CONFIGURATION_DIRECTORY, constants.COMMITTERS), 'r')
    all_lines = committers_file.readlines()
    committers_file.close()
    return all_lines


def _write_committers_to_file(committers: List[str]) -> None:
    committers_file = open(join(CONFIGURATION_DIRECTORY, constants.COMMITTERS), 'w')
    committers_file.writelines(committers)
    committers_file.close()



from typing import List

from guet.commands.command import Command
from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.help.help_message_builder import HelpMessageBuilder
from guet.commands.lambda_strategy import LambdaStrategy
from guet.commands.print_strategy import PrintCommandStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.config.already_initialized import already_initialized
from guet.config.initialize import initialize
from guet.settings.settings import Settings

INIT_HELP_MESSAGE = HelpMessageBuilder('guet init', 'Initialized guet for use on this computer.').build()


class InitCommandFactory(CommandFactoryMethod):

    def short_help_message(self) -> str:
        return 'Initialize guet for use'

    def build(self, args: List[str], settings: Settings) -> Command:
        if not already_initialized():
            return StrategyCommand(LambdaStrategy(lambda: initialize()))
        else:
            return StrategyCommand(PrintCommandStrategy('Config folder already exists.'))



from guet.commands.strategy import CommandStrategy


class InvalidIdentifierStrategy(CommandStrategy):
    def __init__(self, invalid_identifier: str):
        self._invalid_identifier = invalid_identifier

    def apply(self):
        print(f'Invalid identifier <{self._invalid_identifier}>')



from guet.commands.start.hook_strategy import HookStrategy
from guet.git.git import Git


class CreateAlongsideHookStrategy(HookStrategy):
    def apply(self):
        git_path = self.git_path.replace('/hooks', '')
        git = Git(git_path)
        git.create_hooks(alongside=True)



from typing import List

from guet.config.committer import Committer


class SetCommitterObserver:

    def __init__(self):
        pass

    def notify_of_committer_set(self, new_committers: List[Committer]):
        raise NotImplementedError()




class CreateStrategy:
    def apply(self):
        raise NotImplementedError()


class DoCreateStrategy(CreateStrategy):
    def apply(self):
        return True


class DontCreateStrategy(CreateStrategy):
    def apply(self):
        return False



from os import getcwd
from typing import List

from guet.config.committer import Committer
from guet.config.get_current_committers import get_current_committers
from guet.context.context import Context
from guet.util.errors import log_on_error


@log_on_error
def post_commit():
    committers = _rotate_fist_commiter_to_last_committer(get_current_committers())
    context = Context(getcwd())
    context.set_committers(committers)


def _rotate_fist_commiter_to_last_committer(committers: List[Committer]):
    new_last_committer = committers.pop(0)
    committers.append(new_last_committer)
    return committers



from os.path import join
from typing import List

from guet import constants
from guet.config import CONFIGURATION_DIRECTORY
from guet.config.committer import Committer, filter_committers_with_initials
from guet.config.get_committers import get_committers
from guet.files.read_lines import read_lines
from guet.git.git_path_from_cwd import git_path_from_cwd

POSITION_OF_LAST_ELEMENT = -1


def _get_committer_with_initials(committers: List[Committer], initials: str) -> Committer:
    return next(committer for committer in committers if committer.initials == initials)


def _committers_in_order_of_initials(committers: List[Committer],
                                     committer_initials: List[str]) -> List[Committer]:
    return list(map(lambda initials: _get_committer_with_initials(committers, initials), committer_initials))


def _line_ending_with_git_path(lines: List[str]) -> str:
    git_path = git_path_from_cwd()
    return next((line for line in lines if line.endswith(git_path)), None)


def _process_lines_from_committer_set(lines: List[str]) -> List[Committer]:
    line = _line_ending_with_git_path(lines)
    if not line:
        return []
    *committer_initials, _, _ = line.split(',')
    committers = get_committers()
    committers_with_initials = filter_committers_with_initials(committers, committer_initials)
    return _committers_in_order_of_initials(committers_with_initials, committer_initials)


def get_current_committers() -> List[Committer]:
    path = join(CONFIGURATION_DIRECTORY, constants.COMMITTERS_SET)
    lines = read_lines(path)
    if len(lines) > 0:
        return _process_lines_from_committer_set(lines)
    else:
        return []



from os.path import join
from typing import List

from guet.git._create_strategy import CreateStrategy
from guet.git._file_name_strategy import FileNameStrategy
from guet.git.errors import NotGuetHookError
from guet.git.hook import Hook


class HookLoader:
    def __init__(self, path_to_repository, file_name: FileNameStrategy, create: CreateStrategy):
        self.create = create
        self.file_name = file_name
        self.path_to_repository = path_to_repository

    def apply(self, hook_name: str, hooks: List[Hook]):
        hook_path = join(self.path_to_repository, 'hooks', self.file_name.apply(hook_name))
        try:
            hooks.append(Hook(hook_path, create=self.create.apply()))
        except FileNotFoundError:
            pass
        except NotGuetHookError:
            pass



from os.path import join
from typing import List

from guet import constants
from guet.config import CONFIGURATION_DIRECTORY
from guet.config.add_committer import add_committer
from guet.config.committer import Committer
from guet.config.errors import InvalidInitialsError
from guet.config.set_author import set_committer_as_author
from guet.config.set_current_committers import set_current_committers
from guet.context.set_committer_observer import SetCommitterObserver
from guet.files.read_lines import read_lines
from guet.files.write_lines import write_lines


def _load_committers():
    lines = read_lines(join(CONFIGURATION_DIRECTORY, constants.COMMITTERS))
    committers = []
    for line in lines:
        initials, name, email = line.rstrip().split(',')
        committers.append(Committer(initials=initials, name=name, email=email))
    return committers


def _write_committers(committers: List[Committer]):
    write_lines(join(CONFIGURATION_DIRECTORY, constants.COMMITTERS), [str(committer) for committer in committers])


class Committers(SetCommitterObserver):
    def __init__(self):
        super().__init__()
        self._committers = _load_committers()

    def all(self):
        return self._committers

    def add(self, committer: Committer):
        if committer not in self.all():
            self._committers.append(committer)
            add_committer(committer.initials, committer.name, committer.email)

    def by_initials(self, initials: str):
        try:
            return next((committer for committer in self._committers if committer.initials == initials))
        except StopIteration:
            raise InvalidInitialsError()

    def remove(self, committer: Committer):
        self._committers.remove(committer)
        _write_committers(self._committers)

    def notify_of_committer_set(self, new_committers: List[Committer]):
        set_current_committers(new_committers)
        set_committer_as_author(new_committers[0])




def boolean_parser(value: str) -> bool:
    valid_trues = ['True', 'true']
    valid_falses = ['False', 'false']
    if value in valid_trues:
        return True
    elif value in valid_falses:
        return False
    else:
        raise AttributeError



from guet.commands.command import Command
from guet.commands.strategy import CommandStrategy


class StrategyCommand(Command):
    def __init__(self, command_strategy: CommandStrategy):
        super().__init__()
        self.strategy = command_strategy

    def execute(self):
        self.strategy.apply()



from guet.commands.do_nothing_strategy import DoNothingStrategy
from guet.commands.start.create_alongside_hook_strategy import CreateAlongsideHookStrategy
from guet.commands.start.create_hook_strategy import CreateHookStrategy
from guet.commands.start.hook_strategy import HookStrategy


class PromptUserForHookTypeStrategy(HookStrategy):
    def apply(self):
        print(('There is already commit hooks in this project. ' +
               'Would you like to overwrite (o), ' +
               'create (a) the file and put it in the hooks folder, or cancel (x)?'))
        val = input()
        if val == 'o':
            CreateHookStrategy(self.git_path, self.context).apply()
        elif val == 'a':
            CreateAlongsideHookStrategy(self.git_path, self.context).apply()
        else:
            DoNothingStrategy().apply()



from typing import List

from guet.commands.strategy import CommandStrategy
from guet.config.get_config import get_config
from guet.config.set_config import set_config
from guet.context.context import Context
from guet.settings.settings import Settings


def _only_args_starting_with_double_dash(args: List) -> List[str]:
    return [arg for arg in args if arg.startswith('--')]


def _remove_double_dash(key: str) -> str:
    return key.replace('--', '')


def _append_key_and_value_to_config(config: Settings, key_and_value: List):
    for key_value in key_and_value:
        try:
            config.set(key_value[0], key_value[1])
        except KeyError:
            print(f'Cannot set \"{key_value[0]}\", not valid configuration.\n')
            exit(1)


class SetConfigStrategy(CommandStrategy):

    def __init__(self, args: List[str], context: Context):
        self._args = args
        self.context = context

    def apply(self):
        config = get_config()
        key_and_value = self._separate_key_and_value()
        _append_key_and_value_to_config(config, key_and_value)
        set_config(config)

    def _separate_key_and_value(self) -> List:
        key_and_value = []
        for arg in _only_args_starting_with_double_dash(self._args):
            key, value = arg.split('=')
            key_and_value.append((_remove_double_dash(key), value))
        return key_and_value



from guet.hooks.commit_msg import commit_msg as _commit_msg
from guet.hooks.post_commit import post_commit as _post_commit
from guet.hooks.pre_commit import pre_commit as _pre_commit

_HOOK_MAP = {
    '.git/hooks/commit-msg': _commit_msg,
    '.git/hooks/commit-msg-guet': _commit_msg,
    '.git/hooks/pre-commit': _pre_commit,
    '.git/hooks/pre-commit-guet': _pre_commit,
    '.git/hooks/post-commit': _post_commit,
    '.git/hooks/post-commit-guet': _post_commit
}


def manage(hook_filepath: str, *, hook_map=_HOOK_MAP):
    try:
        hook_method = hook_map[hook_filepath]
        hook_method()
    except KeyError:
        pass



from os.path import join
from typing import List

from guet import constants
from guet.config import CONFIGURATION_DIRECTORY
from guet.config.committer import Committer


def get_committers() -> List[Committer]:
    committers_file = open(join(CONFIGURATION_DIRECTORY, constants.COMMITTERS), 'r')
    committers_lines = committers_file.readlines()
    committers_file.close()
    return [_convert_committer_line_to_committer(line) for line in committers_lines]


def _convert_committer_line_to_committer(committer_line: str) -> Committer:
    initials, name, email = committer_line.rstrip().split(',')
    return Committer(initials=initials, name=name, email=email)



from guet.commands.strategy import CommandStrategy


class CancelableCommandStrategy(CommandStrategy):
    def __init__(self, prompt: str, success_callback: CommandStrategy, cancel_callback: CommandStrategy):
        self.prompt = prompt
        self.success_callback = success_callback
        self.cancel_callback = cancel_callback

    def apply(self):
        print(self.prompt)
        answer = input()
        if answer == 'y':
            self.success_callback.apply()
        else:
            if answer != 'x':
                print('Given input invalid. Cancelling command.')
            self.cancel_callback.apply()



from guet.config.errors import PairSetError
from guet.config.get_config import get_config
from guet.config.get_current_committers import get_current_committers
from guet.config.most_recent_committers_set import most_recent_committers_set
from guet.currentmillis import current_millis
from guet.util.errors import log_on_error


@log_on_error
def pre_commit():
    settings = get_config()
    if len(get_current_committers()) == 0:
        _fail_because_there_are_no_current_committers()
    elif settings.read('pairReset'):
        _fail_if_past_valid_timeframe()


def _fail_because_there_are_no_current_committers():
    print('You must set your pairs before you can commit.\n')
    exit(1)


def _fail_if_past_valid_timeframe():
    now = current_millis()
    twenty_four_hours = 86400000
    twenty_four_hours_ago = now - twenty_four_hours
    try:
        set_time = most_recent_committers_set()
        if set_time < twenty_four_hours_ago:
            print(("\nYou have not reset pairs in over twenty four hours!\n" +
                   "Please reset your pairs by using guet set and including your pairs' initials\n"))
            exit(1)
    except PairSetError:
        _fail_because_there_are_no_current_committers()



from typing import List

from guet.commands.command import Command
from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.command_factory_decorator import CommandFactoryDecorator
from guet.commands.help_message_strategy import HelpMessageStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.settings.settings import Settings


class HelpDecorator(CommandFactoryDecorator):

    def __init__(self, decorated: CommandFactoryMethod, help_message: str, no_args_valid=False):
        super().__init__(decorated)
        self._no_args_valid = no_args_valid
        self._help_message = help_message

    def build(self, args: List[str], settings: Settings) -> Command:
        should_print_help_because_of_no_args = len(args) == 1 and len(args) != self._no_args_valid
        if should_print_help_because_of_no_args or '-h' in args or '--help' in args:
            return StrategyCommand(HelpMessageStrategy(self._help_message))
        else:
            return self.decorated.build(args, settings)



from typing import List

from guet.config.committer import Committer
from guet.context.set_committer_observer import SetCommitterObserver


class SetCommittersObservable:
    def __init__(self):
        self.current_committers_observer: List[SetCommitterObserver] = []

    def add_set_committer_observer(self, observer: SetCommitterObserver):
        self.current_committers_observer.append(observer)

    def remove_set_committer_observer(self, observer: SetCommitterObserver):
        self.current_committers_observer.remove(observer)

    def notify_set_committer_observers(self, committers: List[Committer]):
        for observer in self.current_committers_observer:
            observer.notify_of_committer_set(committers)



class Setting:
    def __init__(self, default_value, parser, validator):
        self.default_value = default_value
        self.parser = parser
        self.validator = validator
        self.value = default_value

    def is_default_value(self) -> bool:
        return self.default_value == self.value



from os import getcwd
from os.path import join
from typing import List

from guet.config.committer import Committer
from guet.config.committers import Committers
from guet.context.set_committers_observable import SetCommittersObservable
from guet.git.git import Git
from guet.context.errors import InvalidCommittersError


def _attempt_to_load_git(function):
    def wrapper(*args, **kwargs):
        context: Context = args[0]
        context.__getattribute__('git')
        return function(*args, **kwargs)

    return wrapper


def _attempt_to_load_committers(function):
    def wrapper(*args, **kwargs):
        context: Context = args[0]
        context.__getattribute__('committers')
        return function(*args, **kwargs)

    return wrapper


class Context(SetCommittersObservable):
    @staticmethod
    def instance():
        return Context(getcwd())

    def __init__(self, project_root_directory: str):
        super().__init__()
        self._committers = None
        self._git = None
        self.project_root_directory = project_root_directory

    @property
    def committers(self):
        if self._committers is None:
            self._committers = Committers()
            self.add_set_committer_observer(self._committers)
        return self._committers

    @committers.setter
    def committers(self, new_committers):
        self._committers = new_committers

    @property
    def git(self):
        if self._git is None:
            self._git = Git(join(self.project_root_directory, '.git'))
            self.add_set_committer_observer(self._git)
        return self._git

    @_attempt_to_load_committers
    @_attempt_to_load_git
    def set_committers(self, committers: List[Committer]):
        if len(committers) > 0:
            self.notify_set_committer_observers(committers)
        else:
            raise InvalidCommittersError()



from typing import List


def _append_newline_if_not_present(line: str):
    if line.endswith('\n'):
        return line
    else:
        return f'{line}\n'


def write_lines(path: str, lines: List[str]) -> None:
    file = open(path, 'w')
    file.writelines([_append_newline_if_not_present(line) for line in lines])
    file.close()



from guet.commands.strategy import CommandStrategy
from guet.config.add_committer import add_committer


class AddCommitterStrategy(CommandStrategy):
    def __init__(self, initials: str, name: str, email: str):
        super().__init__()
        self._name = name
        self._email = email
        self._initials = initials

    def apply(self):
        add_committer(self._initials, self._name, self._email)



from typing import List

from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.config.set_config_strategy import SetConfigStrategy
from guet.commands.help.help_message_builder import HelpMessageBuilder
from guet.commands.strategy_command import StrategyCommand
from guet.settings.settings import Settings

CONFIG_HELP_MESSAGE = HelpMessageBuilder('guet config [--<key>=<value> ...', 'Set a configuration.').build()


class ConfigCommandFactory(CommandFactoryMethod):
    def short_help_message(self):
        return 'Change setting values'

    def build(self, args: List[str], settings: Settings):
        return StrategyCommand(SetConfigStrategy(args, self.context))



from typing import List

from guet.git.author import Author


def get_author_lines(config_lines: List[str]):
    user = next((line for line in config_lines if line.startswith('\tname = ')), None)
    email = next((line for line in config_lines if line.startswith('\temail = ')), None)
    return user, email


def load_author(config_lines: List[str]) -> Author:
    user, email = get_author_lines(config_lines)
    if user is None or email is None:
        return None
    else:
        return Author(name=user.replace('\tname = ', ''), email=email.replace('\temail = ', ''))


def overwrite_current_author(current_config: List[str], new_author: Author) -> None:
    for i, line in enumerate(current_config):
        if 'name =' in line:
            current_config[i] = f'\tname = {new_author.name}'
        elif 'email =' in line:
            current_config[i] = f'\temail = {new_author.email}'


def append_new_author(new_lines: List[str], new_author: Author):
    new_lines.append('[user]')
    new_lines.append(f'\tname = {new_author.name}')
    new_lines.append(f'\temail = {new_author.email}')



import guet
from guet.commands.command import Command
from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.help.guet_usage import guet_usage
from guet.commands.help_message_strategy import HelpMessageStrategy
from guet.commands.print_strategy import PrintCommandStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.settings.settings import Settings
from guet.config.get_config import get_config
from guet.config.already_initialized import already_initialized


class Executor:
    def __init__(self, command_builder_map):
        self.command_builder_map = command_builder_map

    def create(self, args: list) -> Command:
        if already_initialized():
            return self._create_with_settings(args, get_config())
        else:
            return self._create_with_settings(args, Settings())

    def _create_with_settings(self, args: list, settings: Settings) -> Command:
        if len(args) > 0:
            command_arg = args[0]
            if command_arg in ('--version', '-v'):
                return StrategyCommand(PrintCommandStrategy(guet.__version__))
            return self.load_command_factory_from_map(args, settings)
        else:
            return self._guet_ussage_command()

    def load_command_factory_from_map(self, args, settings):
        command_arg = args[0]
        try:
            command_factory: CommandFactoryMethod = self.command_builder_map[command_arg]
            return command_factory.build(args, settings)
        except KeyError:
            return self.load_invalid_command(command_arg)

    def load_invalid_command(self, command_arg: str):
        if command_arg in ('-h', '--help'):
            return self._guet_ussage_command()
        return self._guet_ussage_command(invalid_command=command_arg)

    def _guet_ussage_command(self, *, invalid_command: str = ''):
        message = guet_usage(self.command_builder_map)
        if invalid_command:
            message = f'guet has no command "{invalid_command}" that can be ran.\n\n' + message
        return StrategyCommand(HelpMessageStrategy(message))



import unittest
from unittest.mock import patch, Mock
import guet
from guet.commands.help.guet_usage import guet_usage
from guet.executor import Executor


@patch('guet.executor.already_initialized', return_value=True)
@patch('guet.executor.get_config')
class TestCommandFactory(unittest.TestCase):

    def test_returns_command_with_settings_from_settings_file(self, mock_get_settings,
                                                              mock_already_init):
        builder_map = dict()
        mock_factory = Mock()
        builder_map['command'] = mock_factory

        command_factory = Executor(builder_map)
        args = ['command']
        command_factory.create(args)
        mock_factory.build.assert_called_with(args, mock_get_settings.return_value)

    @patch('builtins.print')
    def test_uses_command_builder_map_to_print_help_messages(self, mock_print, mock_get_settings,
                                                             mock_already_init):
        builder_map = dict()
        builder_map['command'] = Mock()
        command_factory = Executor(builder_map)
        result = command_factory.create([])
        result.execute()
        mock_print.assert_called_with(guet_usage(builder_map))

    @patch('builtins.print')
    def test_returns_command_that_prints_version_when_given_dash_dash_version_flag(self, mock_print,
                                                                                   mock_get_settings,
                                                                                   mock_already_init):
        builder_map = dict()
        builder_map['command'] = Mock()

        command_factory = Executor(builder_map)

        result = command_factory.create(['--version'])
        result.execute()
        mock_print.assert_called_with(f'{guet.__version__}')

    @patch('builtins.print')
    def test_returns_command_that_prints_version_when_given_dash_v(self, mock_print,
                                                                   mock_get_settings,
                                                                   mock_already_init):
        builder_map = dict()
        builder_map['command'] = Mock()

        command_factory = Executor(builder_map)

        result = command_factory.create(['-v'])
        result.execute()
        mock_print.assert_called_with(f'{guet.__version__}')

    @patch('builtins.print')
    def test_returns_command_that_prints_usage(self, mock_print,
                                               mock_get_settings,
                                               mock_already_init):
        builder_map = dict()
        builder_map['command'] = Mock()

        command_factory = Executor(builder_map)

        result = command_factory.create(['-h'])
        result.execute()
        mock_print.assert_called_with(guet_usage(builder_map))



class FileNameStrategy:
    def apply(self, base_name: str) -> str:
        raise NotImplementedError()


class AlongsideFileNameStrategy(FileNameStrategy):
    def apply(self, base_name: str):
        return base_name + '-guet'


class BaseFileNameStrategy(FileNameStrategy):
    def apply(self, base_name: str) -> str:
        return base_name



from guet.commands.strategy import CommandStrategy


class HelpMessageStrategy(CommandStrategy):
    def __init__(self, help_message: str):
        self.help_message = help_message

    def apply(self):
        print(self.help_message)



from typing import List

from guet.commands.command import Command
from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.get.committer_printing_strategy import CommitterPrintingStrategy
from guet.commands.get.invalid_identifier_strategy import InvalidIdentifierStrategy
from guet.commands.help.help_message_builder import HelpMessageBuilder, FlagBuilder, FlagsBuilder
from guet.commands.strategy import CommandStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.config.committer import Committer
from guet.config.get_current_committers import get_current_committers
from guet.settings.settings import Settings


def print_full_names(committers: List[Committer]) -> None:
    for committer in committers:
        print(committer.pretty())


def print_only_initials(committers: List[Committer]) -> None:
    print(', '.join([committer.initials for committer in committers]))


GET_HELP_MESSAGE = HelpMessageBuilder('guet get <identifier> [-flag, ...]', 'Get currently set information.') \
    .explanation('Valid Identifier\n\n\tcurrent - lists currently set committers\n\tcomitters - lists all committers') \
    .flags(FlagsBuilder([FlagBuilder('l', 'Print values as truncated list')])).build()


class GetCommandFactory(CommandFactoryMethod):
    def short_help_message(self):
        return 'Get information about the current state of the system'

    def build(self, args: List[str], settings: Settings) -> Command:
        try:
            strategy = self._determine_strategy(args)
            return StrategyCommand(strategy)
        except AttributeError:
            return StrategyCommand(InvalidIdentifierStrategy(args[0]))

    def _determine_identifier(self, identifier: str):
        if identifier == 'committers':
            return self.context.committers.all(), lambda: print('All committers')
        elif identifier == 'current':
            return get_current_committers(), lambda: print('Currently set committers')
        raise AttributeError

    def _determine_strategy(self, args: List[str]) -> CommandStrategy:
        identifier = args[1]
        committers, pre_print = self._determine_identifier(identifier)
        if '-l' in args:
            return CommitterPrintingStrategy(committers, lambda: None, print_only_initials)
        else:
            return CommitterPrintingStrategy(committers, pre_print, print_full_names)



APP_FOLDER_NAME = '.guet'
DATA_SOURCE_NAME = 'guet.db'
COMMITTER_NAMES = 'committernames'
AUTHOR_NAME = 'authornames'
AUTHOR_EMAIL = 'authoremails'
COMMITTERS = 'committers'
COMMITTERS_SET = 'committersset'
CONFIG = 'config'
ERRORS = 'errors'



from typing import List

from guet.commands.command import Command
from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.help.help_message_builder import HelpMessageBuilder, FlagBuilder, FlagsBuilder
from guet.commands.start.create_alongside_hook_strategy import CreateAlongsideHookStrategy
from guet.commands.start.create_hook_strategy import CreateHookStrategy
from guet.commands.start.start_strategy import PromptUserForHookTypeStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.git.git import Git
from guet.git.git_path_from_cwd import git_path_from_cwd
from guet.settings.settings import Settings

START_HELP_MESSAGE = HelpMessageBuilder('guet start',
                                        'Initialize current .git project to use guet.') \
    .flags(FlagsBuilder([FlagBuilder('-a/--alongside', 'Create hooks alongside current hooks with "-guet" on the end'),
                         FlagBuilder('-o/--overwrite', 'Overwrite current hooks')])).build()


class StartCommandFactory(CommandFactoryMethod):
    def short_help_message(self) -> str:
        return 'Start guet usage in the repository at current directory'

    def build(self, args: List[str], settings: Settings) -> Command:
        git_path = git_path_from_cwd().replace('/hooks', '')
        git = Git(git_path)
        if '-a' in args or '--alongside' in args:
            strategy = CreateAlongsideHookStrategy(git_path, self.context)
        elif '-o' in args or '--overwrite' in args:
            strategy = CreateHookStrategy(git_path, self.context)
        elif git.non_guet_hooks_present():
            strategy = PromptUserForHookTypeStrategy(git_path, self.context)
        else:
            strategy = CreateHookStrategy(git_path, self.context)
        return StrategyCommand(strategy)



from typing import List

from guet.commands.command import Command
from guet.commands.command_factory_decorator import CommandFactoryDecorator
from guet.commands.print_strategy import PrintCommandStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.git.git import Git
from guet.git.git_path_from_cwd import git_path_from_cwd
from guet.settings.settings import Settings

GUET_NOT_STARTED_ERROR = (
    'guet not initialized in this repository. '
    'Please use guet start to initialize repository '
    'for use with guet.'
)


class StartRequiredDecorator(CommandFactoryDecorator):
    def build(self, args: List[str], settings: Settings) -> Command:
        git = Git(git_path_from_cwd())
        if git.hooks_present():
            return self.decorated.build(args, settings)
        else:
            return StrategyCommand(PrintCommandStrategy(GUET_NOT_STARTED_ERROR))



from guet.commands.command_factory import CommandFactoryMethod


def guet_usage(command_builder_map: dict) -> str:
    help_message = 'usage: guet <command>\n'
    for key in command_builder_map:
        short_help_message = ''
        if not isinstance(command_builder_map[key], CommandFactoryMethod):
            short_help_message = command_builder_map[key].get_short_help_message()
        else:
            short_help_message = command_builder_map[key].short_help_message()
        help_message += f'\n   {key} -- {short_help_message}'
    return help_message + '\n'



from os import mkdir
from os.path import join

from guet import constants, __version__
from guet.config import CONFIGURATION_DIRECTORY
from guet.files.write_lines import write_lines


def _prepend_version_number_to_config_file():
    write_lines(join(CONFIGURATION_DIRECTORY, constants.CONFIG), [f'{__version__}\n', '\n'])


def initialize():
    mkdir(CONFIGURATION_DIRECTORY)
    _create_file_with_name(join(CONFIGURATION_DIRECTORY, constants.COMMITTER_NAMES))
    _create_file_with_name(join(CONFIGURATION_DIRECTORY, constants.AUTHOR_NAME))
    _create_file_with_name(join(CONFIGURATION_DIRECTORY, constants.AUTHOR_EMAIL))
    _create_file_with_name(join(CONFIGURATION_DIRECTORY, constants.COMMITTERS))
    _create_file_with_name(join(CONFIGURATION_DIRECTORY, constants.COMMITTERS_SET))
    _create_file_with_name(join(CONFIGURATION_DIRECTORY, constants.CONFIG))
    _create_file_with_name(join(CONFIGURATION_DIRECTORY, constants.ERRORS))
    _prepend_version_number_to_config_file()


def _create_file_with_name(file_name: str) -> None:
    open(file_name, 'w').close()



from guet.commands.strategy import CommandStrategy
from guet.context.context import Context


class HookStrategy(CommandStrategy):
    def __init__(self, git_path: str, context: Context):
        self.git_path = git_path
        self.context = context

    def apply(self):
        raise NotImplementedError



from os.path import join

from guet import constants
from guet.config import CONFIGURATION_DIRECTORY
from guet.files.read_lines import read_lines
from guet.settings.settings import Settings


def get_config() -> Settings:
    lines = read_lines(join(CONFIGURATION_DIRECTORY, constants.CONFIG))
    settings = Settings()
    settings.load(lines)
    return settings



from typing import List


class _Builder:

    def build(self) -> str:
        raise NotImplementedError


class _NoneSection(_Builder):

    def build(self) -> str:
        return ''


class _SectionBuild(_Builder):
    def __init__(self, s: str):
        self._str = s

    def build(self) -> str:
        return f'\n{self._str}\n'


class FlagBuilder(_Builder):
    def __init__(self, key: str, value: str):
        self._key = key
        self._value = value

    def build(self) -> str:
        return f'\t{self._key}  -  {self._value}'


class FlagsBuilder(_Builder):
    def __init__(self, flags: List[FlagBuilder]):
        self._flags = flags

    def build(self) -> str:
        flags = "\n".join([f.build() for f in self._flags])
        return f'\nFlags\n{flags}\n'


class HelpMessageBuilder(_Builder):

    def __init__(self, usage: str, description: str):
        self._usage: str = usage
        self._description: _Builder = _SectionBuild(description)
        self._explanation: _Builder = _NoneSection()
        self._flags: _Builder = _NoneSection()

    def explanation(self, explanation: str):
        self._explanation = _SectionBuild(explanation)
        return self

    def flags(self, flags_builder: FlagsBuilder):
        self._flags = flags_builder
        return self

    def build(self) -> str:
        message = (f'usage: {self._usage}\n'
                   f'{self._description.build()}'
                   f'{self._explanation.build()}'
                   f'{self._flags.build()}'
                   f'\n')
        return message



from typing import List

from guet.commands.command import Command
from guet.context.context import Context
from guet.settings.settings import Settings


class CommandFactoryMethod:
    def __init__(self):
        self.context = Context.instance()

    def short_help_message(self) -> str:
        raise NotImplementedError

    def build(self, args: List[str], settings: Settings) -> Command:
        raise NotImplementedError



from typing import List

from guet.commands.command import Command
from guet.commands.command_factory import CommandFactoryMethod
from guet.commands.help.help_message_builder import HelpMessageBuilder
from guet.commands.remove.remove_strategy import RemoveCommitterStrategy
from guet.commands.strategy_command import StrategyCommand
from guet.settings.settings import Settings

REMOVE_HELP_MESSAGE = HelpMessageBuilder('guet remove <initials>', ('Remove committer '
                                                                    'with given initials from system.')).build()


class RemoveCommandFactory(CommandFactoryMethod):

    def short_help_message(self) -> str:
        return 'Removes committer'

    def build(self, args: List[str], settings: Settings) -> Command:
        return StrategyCommand(RemoveCommitterStrategy(args[1], self.context))



from guet.commands.strategy import CommandStrategy


class TooFewArgsStrategy(CommandStrategy):
    def __init__(self, help_message):
        super().__init__()
        self._help_message = help_message

    def apply(self):
        print('Not enough arguments.')
        print('')
        print(self._help_message)



from guet.commands.strategy import CommandStrategy
from guet.config.errors import InvalidInitialsError
from guet.context.context import Context


def _without_committer_with_initials(initials, all_committers):
    return list(filter(lambda committer: committer.initials != initials, all_committers))


class RemoveCommitterStrategy(CommandStrategy):
    def __init__(self, initials: str, context: Context):
        self._initials = initials
        self.context = context

    def apply(self):
        try:
            committer_to_be_removed = self.context.committers.by_initials(self._initials)
            self.context.committers.remove(committer_to_be_removed)
        except InvalidInitialsError:
            print(f'No committer exists with initials "{self._initials}"')



import time
from os.path import join
from typing import List

from guet import constants
from guet.config import CONFIGURATION_DIRECTORY
from guet.config.committer import Committer
from guet.files.write_lines import write_lines
from guet.files.read_lines import read_lines
from guet.git.git_path_from_cwd import git_path_from_cwd


def _all_committers_set_from_file() -> List[str]:
    return read_lines(join(CONFIGURATION_DIRECTORY, constants.COMMITTERS_SET))


def _format_committers_to_committers_set_format(committers: List[Committer]) -> str:
    git_path = git_path_from_cwd()
    current_time_in_millis = int(round(time.time() * 1000))
    committer_initials = ','.join([committer.initials for committer in committers])
    return committer_initials + f',{current_time_in_millis},{git_path}'


def _add_to_current_set_lines(current_set, formatted_set_committers_information):
    git_path = git_path_from_cwd()
    line_with_git_path = next((line for line in current_set if line.endswith(git_path)), None)
    if line_with_git_path:
        index = current_set.index(line_with_git_path)
        current_set[index] = formatted_set_committers_information
    else:
        current_set.append(formatted_set_committers_information)


def set_current_committers(committers: List[Committer]) -> None:
    path = join(CONFIGURATION_DIRECTORY, constants.COMMITTERS_SET)
    current_set = _all_committers_set_from_file()
    formatted_set_committers_information = _format_committers_to_committers_set_format(committers)
    _add_to_current_set_lines(current_set, formatted_set_committers_information)
    write_lines(path, current_set)



import sys

from guet.commands.get.get_factory import GetCommandFactory, GET_HELP_MESSAGE
from guet.commands.git_required_decorator import GitRequiredDecorator
from guet.commands.help_decorator import HelpDecorator
from guet.commands.init_required_decorator import InitRequiredDecorator
from guet.commands.remove.factory import RemoveCommandFactory, REMOVE_HELP_MESSAGE
from guet.commands.start.factory import StartCommandFactory, START_HELP_MESSAGE
from guet.commands.start_required_decorator import StartRequiredDecorator
from guet.commands.version_decorator import VersionDecorator
from guet.executor import Executor
from guet.util.errors import log_on_error
from guet.commands.addcommitter.factory import AddCommitterFactory, ADD_COMMITTER_HELP_MESSAGE
from guet.commands.init.factory import InitCommandFactory, INIT_HELP_MESSAGE
from guet.commands.setcommitters.factory import SetCommittersCommandFactory, SET_HELP_MESSAGE
from guet.commands.config.factory import ConfigCommandFactory, CONFIG_HELP_MESSAGE


def _command_builder_map():
    command_builder_map = dict()
    command_builder_map['add'] = VersionDecorator(
        InitRequiredDecorator(HelpDecorator(AddCommitterFactory(), ADD_COMMITTER_HELP_MESSAGE)))

    command_builder_map['init'] = VersionDecorator(
        HelpDecorator(InitCommandFactory(), INIT_HELP_MESSAGE, no_args_valid=True))

    command_builder_map['set'] = VersionDecorator(InitRequiredDecorator(
        StartRequiredDecorator(HelpDecorator(SetCommittersCommandFactory(), SET_HELP_MESSAGE))))

    command_builder_map['start'] = VersionDecorator(InitRequiredDecorator(
        HelpDecorator(
            GitRequiredDecorator(StartCommandFactory()), START_HELP_MESSAGE, no_args_valid=True
        )
    ))
    command_builder_map['config'] = VersionDecorator(
        InitRequiredDecorator(HelpDecorator(ConfigCommandFactory(), CONFIG_HELP_MESSAGE)))

    command_builder_map['get'] = VersionDecorator(
        InitRequiredDecorator(HelpDecorator(GetCommandFactory(), GET_HELP_MESSAGE)))

    command_builder_map['remove'] = VersionDecorator(
        InitRequiredDecorator(HelpDecorator(RemoveCommandFactory(), REMOVE_HELP_MESSAGE)))

    return command_builder_map


@log_on_error
def main():
    command_factory = Executor(_command_builder_map())
    command = command_factory.create(sys.argv[1:])
    command.execute()



from os.path import join

from guet import constants
from guet.config import CONFIGURATION_DIRECTORY
from guet.config.committer import Committer


def set_committer_as_author(committer: Committer):
    author_name_file = open(join(CONFIGURATION_DIRECTORY, constants.AUTHOR_NAME), 'w')
    author_name_file.write(f'{committer.name}\n')
    author_name_file.close()
    author_email_file = open(join(CONFIGURATION_DIRECTORY, constants.AUTHOR_EMAIL), 'w')
    author_email_file.write(f'{committer.email}\n')
    author_email_file.close()
